@@ -45,6 +45,7 @@ int main_vcfsom(int argc, char *argv[]);
 int main_vcfnorm(int argc, char *argv[]);
 int main_vcfgtcheck(int argc, char *argv[]);
 int main_vcfview(int argc, char *argv[]);
+int main_vcfhead(int argc, char *argv[]);
 int main_vcfcall(int argc, char *argv[]);
 int main_vcfannotate(int argc, char *argv[]);
 int main_vcfroh(int argc, char *argv[]);
@@ -57,6 +58,7 @@ int main_polysomy(int argc, char *argv[]);
 #endif
 #ifdef ENABLE_BCF_PLUGINS
 int main_plugin(int argc, char *argv[]);
+int count_plugins(void);
 #endif
 int main_consensus(int argc, char *argv[]);
 int main_csq(int argc, char *argv[]);
@@ -102,6 +104,10 @@ static cmd_t cmds[] =
       .alias = "convert",
       .help  = "convert VCF/BCF files to different formats and back"
     },
+    { .func  = main_vcfhead,
+      .alias = "head",
+      .help  = "view VCF/BCF file headers"
+    },
     { .func  = main_vcfisec,
       .alias = "isec",
       .help  = "intersections of VCF/BCF files"
@@ -227,6 +233,14 @@ static void usage(FILE *fp)
         if ( cmds[i].func && cmds[i].help[0]!='-' ) fprintf(fp, "    %-12s %s\n", cmds[i].alias, cmds[i].help);
         i++;
     }
+#if ENABLE_BCF_PLUGINS
+    fprintf(fp,"\n -- Plugins (collection of programs for calling, file manipulation & analysis)\n");
+    int nplugins = count_plugins();
+    if ( nplugins )
+        fprintf(fp,"    %d plugins available, run \"bcftools plugin -lv\" to see a complete list\n", nplugins);
+    else
+        fprintf(fp,"    0 plugins available, run \"bcftools plugin -l\" for help\n");
+#endif
     fprintf(fp,"\n");
     fprintf(fp,
             " Most commands accept VCF, bgzipped VCF, and BCF with the file type detected\n"
@@ -253,7 +267,7 @@ int bcftools_main(int argc, char *argv[])
     if (argc < 2) { usage(bcftools_stderr); return 1; }
 
     if (strcmp(argv[1], "version") == 0 || strcmp(argv[1], "--version") == 0 || strcmp(argv[1], "-v") == 0) {
-        fprintf(bcftools_stdout, "bcftools %s\nUsing htslib %s\nCopyright (C) 2021 Genome Research Ltd.\n", bcftools_version(), hts_version());
+        fprintf(bcftools_stdout, "bcftools %s\nUsing htslib %s\nCopyright (C) 2022 Genome Research Ltd.\n", bcftools_version(), hts_version());
 #if USE_GPL
         fprintf(bcftools_stdout, "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n");
 #else
