@@ -139,7 +139,7 @@ typedef struct _args_t
     bcf_srs_t *files;
     bcf_hdr_t *hdr, *hdr_out;
     htsFile *out_fh;
-    int output_type, n_threads;
+    int output_type, n_threads, clevel;
 
     filter_t *filter;
     char *filter_str;
@@ -457,11 +457,22 @@ static int cmp_plugin_name(const void *p1, const void *p2)
     return strcmp(a->name,b->name);
 }
 
+// If args=NULL then returns the number of plugins available. Otherwise prints the
+// plugins on bcftools_stdout and returns 0 on success.
 static int list_plugins(args_t *args)
 {
     plugin_t *plugins = NULL;
     int nplugins = 0, mplugins = 0;
 
+    int count_only = 0;
+    args_t _args;
+    if ( !args )
+    {
+        memset(&_args,0,sizeof(_args));
+        args = &_args;
+        args->nplugin_paths = -1;
+        count_only = 1;
+    }
     init_plugin_paths(args);
 
     kstring_t str = {0,0,0};
@@ -492,6 +503,11 @@ static int list_plugins(args_t *args)
         }
         closedir(dp);
     }
+    if ( count_only )
+    {
+        free(str.s);
+        return nplugins;
+    }
     if ( nplugins )
     {
         qsort(plugins, nplugins, sizeof(plugins[0]), cmp_plugin_name);
@@ -510,6 +526,10 @@ static int list_plugins(args_t *args)
     free(str.s);
     return nplugins ? 0 : 1;
 }
+int count_plugins(void)
+{
+    return list_plugins(NULL);
+}
 
 static void init_data(args_t *args)
 {
@@ -524,7 +544,9 @@ static void init_data(args_t *args)
     if (args->record_cmd_line) bcf_hdr_append_version(args->hdr_out, args->argc, args->argv, "bcftools_plugin");
     if ( !args->drop_header )
     {
-        args->out_fh = hts_open(args->output_fname,hts_bcf_wmode2(args->output_type,args->output_fname));
+        char wmode[8];
+        set_wmode(wmode,args->output_type,args->output_fname,args->clevel);
+        args->out_fh = hts_open(args->output_fname ? args->output_fname : "-", wmode);
         if ( args->out_fh == NULL ) error("Can't write to \"%s\": %s\n", args->output_fname, strerror(errno));
         if ( args->n_threads ) hts_set_threads(args->out_fh, args->n_threads);
         if ( bcf_hdr_write(args->out_fh, args->hdr_out)!=0 ) error("[%s] Error: cannot write to %s\n", __func__,args->output_fname);
@@ -560,22 +582,24 @@ static void usage(args_t *args)
     fprintf(bcftools_stderr, "         bcftools +name [OPTIONS] <file>  [-- PLUGIN_OPTIONS]\n");
     fprintf(bcftools_stderr, "\n");
     fprintf(bcftools_stderr, "VCF input options:\n");
-    fprintf(bcftools_stderr, "   -e, --exclude <expr>        exclude sites for which the expression is true\n");
-    fprintf(bcftools_stderr, "   -i, --include <expr>        select sites for which the expression is true\n");
-    fprintf(bcftools_stderr, "   -r, --regions <region>      restrict to comma-separated list of regions\n");
-    fprintf(bcftools_stderr, "   -R, --regions-file <file>   restrict to regions listed in a file\n");
-    fprintf(bcftools_stderr, "   -t, --targets <region>      similar to -r but streams rather than index-jumps\n");
-    fprintf(bcftools_stderr, "   -T, --targets-file <file>   similar to -R but streams rather than index-jumps\n");
+    fprintf(bcftools_stderr, "   -e, --exclude EXPR             Exclude sites for which the expression is true\n");
+    fprintf(bcftools_stderr, "   -i, --include EXPR             Select sites for which the expression is true\n");
+    fprintf(bcftools_stderr, "   -r, --regions REGION           Restrict to comma-separated list of regions\n");
+    fprintf(bcftools_stderr, "   -R, --regions-file FILE        Restrict to regions listed in a file\n");
+    fprintf(bcftools_stderr, "        --regions-overlap 0|1|2   Include if POS in the region (0), record overlaps (1), variant overlaps (2) [1]\n");
+    fprintf(bcftools_stderr, "   -t, --targets REGION           Similar to -r but streams rather than index-jumps\n");
+    fprintf(bcftools_stderr, "   -T, --targets-file FILE        Similar to -R but streams rather than index-jumps\n");
+    fprintf(bcftools_stderr, "        --targets-overlap 0|1|2   Include if POS in the region (0), record overlaps (1), variant overlaps (2) [0]\n");
     fprintf(bcftools_stderr, "VCF output options:\n");
-    fprintf(bcftools_stderr, "       --no-version            do not append version and command line to the header\n");
-    fprintf(bcftools_stderr, "   -o, --output <file>         write output to a file [standard output]\n");
-    fprintf(bcftools_stderr, "   -O, --output-type <type>    'b' compressed BCF; 'u' uncompressed BCF; 'z' compressed VCF; 'v' uncompressed VCF [v]\n");
-    fprintf(bcftools_stderr, "       --threads <int>         use multithreading with <int> worker threads [0]\n");
+    fprintf(bcftools_stderr, "       --no-version               Do not append version and command line to the header\n");
+    fprintf(bcftools_stderr, "   -o, --output FILE              Write output to a file [standard output]\n");
+    fprintf(bcftools_stderr, "   -O, --output-type u|b|v|z[0-9] u/b: un/compressed BCF, v/z: un/compressed VCF, 0-9: compression level [v]\n");
+    fprintf(bcftools_stderr, "       --threads INTT             Use multithreading with <int> worker threads [0]\n");
     fprintf(bcftools_stderr, "Plugin options:\n");
-    fprintf(bcftools_stderr, "   -h, --help                  list plugin's options\n");
-    fprintf(bcftools_stderr, "   -l, --list-plugins          list available plugins. See BCFTOOLS_PLUGINS environment variable and man page for details\n");
-    fprintf(bcftools_stderr, "   -v, --verbose               print verbose information, -vv increases verbosity\n");
-    fprintf(bcftools_stderr, "   -V, --version               print version string and exit\n");
+    fprintf(bcftools_stderr, "   -h, --help                     List plugin's options\n");
+    fprintf(bcftools_stderr, "   -l, --list-plugins             List available plugins. See BCFTOOLS_PLUGINS environment variable and man page for details\n");
+    fprintf(bcftools_stderr, "   -v, --verbose                  Print verbose information, -vv increases verbosity\n");
+    fprintf(bcftools_stderr, "   -V, --version                  Print version string and exit\n");
     fprintf(bcftools_stderr, "\n");
     bcftools_exit(1);
 }
@@ -611,7 +635,10 @@ int main_plugin(int argc, char *argv[])
     args->n_threads = 0;
     args->record_cmd_line = 1;
     args->nplugin_paths = -1;
+    args->clevel = -1;
     int regions_is_file = 0, targets_is_file = 0, usage_only = 0, version_only = 0;
+    int regions_overlap = 1;
+    int targets_overlap = 0;
 
     if ( argc==1 ) usage(args);
     char *plugin_name = NULL;
@@ -645,11 +672,14 @@ int main_plugin(int argc, char *argv[])
         {"exclude",required_argument,NULL,'e'},
         {"regions",required_argument,NULL,'r'},
         {"regions-file",required_argument,NULL,'R'},
+        {"regions-overlap",required_argument,NULL,1},
         {"targets",required_argument,NULL,'t'},
         {"targets-file",required_argument,NULL,'T'},
+        {"targets-overlap",required_argument,NULL,2},
         {"no-version",no_argument,NULL,8},
         {NULL,0,NULL,0}
     };
+    char *tmp;
     while ((c = getopt_long(argc, argv, "h?o:O:r:R:t:T:li:e:vV",loptions,NULL)) >= 0)
     {
         switch (c) {
@@ -662,8 +692,17 @@ int main_plugin(int argc, char *argv[])
                     case 'u': args->output_type = FT_BCF; break;
                     case 'z': args->output_type = FT_VCF_GZ; break;
                     case 'v': args->output_type = FT_VCF; break;
-                    default: error("The output type \"%s\" not recognised\n", optarg);
+                    default:
+                    {
+                        args->clevel = strtol(optarg,&tmp,10);
+                        if ( *tmp || args->clevel<0 || args->clevel>9 ) error("The output type \"%s\" not recognised\n", optarg);
+                    }
                 };
+                if ( optarg[1] )
+                {
+                    args->clevel = strtol(optarg+1,&tmp,10);
+                    if ( *tmp || args->clevel<0 || args->clevel>9 ) error("Could not parse argument: --compression-level %s\n", optarg+1);
+                }
                 break;
             case 'e':
                 if ( args->filter_str ) error("Error: only one -i or -e expression can be given, and they cannot be combined\n");
@@ -676,6 +715,14 @@ int main_plugin(int argc, char *argv[])
             case 't': args->targets_list = optarg; break;
             case 'T': args->targets_list = optarg; targets_is_file = 1; break;
             case 'l': args->plist_only = 1; break;
+            case  1 :
+                regions_overlap = parse_overlap_option(optarg);
+                if ( regions_overlap < 0 ) error("Could not parse: --regions-overlap %s\n",optarg);
+                break;
+            case  2 :
+                targets_overlap = parse_overlap_option(optarg);
+                if ( targets_overlap < 0 ) error("Could not parse: --targets-overlap %s\n",optarg);
+                break;
             case  9 : args->n_threads = strtol(optarg, 0, 0); break;
             case  8 : args->record_cmd_line = 0; break;
             case '?':
@@ -729,11 +776,13 @@ int main_plugin(int argc, char *argv[])
     args->files = bcf_sr_init();
     if ( args->regions_list )
     {
+        bcf_sr_set_opt(args->files,BCF_SR_REGIONS_OVERLAP,regions_overlap);
         if ( bcf_sr_set_regions(args->files, args->regions_list, regions_is_file)<0 )
             error("Failed to read the regions: %s\n", args->regions_list);
     }
     if ( args->targets_list )
     {
+        bcf_sr_set_opt(args->files,BCF_SR_TARGETS_OVERLAP,targets_overlap);
         if ( bcf_sr_set_targets(args->files, args->targets_list, targets_is_file, 0)<0 )
             error("Failed to read the targets: %s\n", args->targets_list);
         args->files->collapse |= COLLAPSE_SOME;
