@@ -1,7 +1,7 @@
 /*  bam2depth.c -- depth subcommand.
 
     Copyright (C) 2011, 2012 Broad Institute.
-    Copyright (C) 2012-2016, 2018, 2019-2021 Genome Research Ltd.
+    Copyright (C) 2012-2016, 2018, 2019-2022 Genome Research Ltd.
 
     Author: Heng Li <lh3@sanger.ac.uk> (to 2020)
     Author: James Bonfield <jkb@sanger.ac.uk> (2021 rewrite)
@@ -717,12 +717,14 @@ static void usage_exit(FILE *fp, int exit_status)
     fprintf(fp, "  -H           Print a file header line\n");
     fprintf(fp, "  -l INT       Minimum read length [0]\n");
     fprintf(fp, "  -o FILE      Write output to FILE [stdout]\n");
-    fprintf(fp, "  -q INT       Minimum base quality [0]\n");
-    fprintf(fp, "  -Q INT       Minimum mapping quality [0]\n");
+    fprintf(fp, "  -q, --min-BQ INT\n"
+                "               Filter bases with base quality smaller than INT [0]\n");
+    fprintf(fp, "  -Q, --min-MQ INT\n"
+                "               Filter alignments with mapping quality smaller than INT [0]\n");
     fprintf(fp, "  -H           Print a file header\n");
     fprintf(fp, "  -J           Include reads with deletions in depth computation\n");
     fprintf(fp, "  -s           Do not count overlapping reads within a template\n");
-    sam_global_opt_help(fp, "-.---@-.");
+    sam_global_opt_help(fp, "-.--.@-.");
     exit(exit_status);
 }
 
@@ -733,6 +735,7 @@ int main_depth(int argc, char *argv[])
     sam_hdr_t **header;
     int c, has_index_file = 0;
     char *file_list = NULL, **fn = NULL;
+    char *out_file = NULL;
     depth_opt opt = {
         .flag = BAM_FUNMAP | BAM_FSECONDARY | BAM_FDUP | BAM_FQCFAIL,
         .min_qual = 0,
@@ -749,7 +752,11 @@ int main_depth(int argc, char *argv[])
 
     sam_global_args ga = SAM_GLOBAL_ARGS_INIT;
     static const struct option lopts[] = {
-        SAM_OPT_GLOBAL_OPTIONS('-', 0, '-', '-', '-', '@'),
+        {"min-MQ", required_argument, NULL, 'Q'},
+        {"min-mq", required_argument, NULL, 'Q'},
+        {"min-BQ", required_argument, NULL, 'q'},
+        {"min-bq", required_argument, NULL, 'q'},
+        SAM_OPT_GLOBAL_OPTIONS('-', 0, '-', '-', 0, '@'),
         {NULL, 0, NULL, 0}
     };
 
@@ -807,7 +814,7 @@ int main_depth(int argc, char *argv[])
         case 'o':
             if (opt.out != stdout)
                 break;
-            opt.out = fopen(optarg, "w");
+            opt.out = fopen(out_file = optarg, "w");
             if (!opt.out) {
                 print_error_errno("depth", "Cannot open \"%s\" for writing.",
                                   optarg);
@@ -948,7 +955,13 @@ int main_depth(int argc, char *argv[])
     if (opt.bed)
         bed_destroy(opt.bed);
     sam_global_args_free(&ga);
-    if (opt.out != stdout) fclose(opt.out);
+    if (opt.out != stdout) {
+        if (fclose(opt.out) != 0 && ret == 0) {
+            print_error_errno("depth", "error on closing \"%s\"", out_file);
+            ret = 1;
+        }
+    }
+
     return ret;
 }
 
