@@ -32,9 +32,21 @@ DEALINGS IN THE SOFTWARE.  */
 
 #include "samtools.h"
 
+static htsFile *samtools_stdout = NULL;
+
+void autoflush_if_stdout(htsFile *fp, const char *fname) {
+    if (fname == NULL || strcmp(fname, "-") == 0) samtools_stdout = fp;
+}
+
+void release_autoflush(htsFile *fp) {
+    if (samtools_stdout == fp) samtools_stdout = NULL;
+}
+
 static void vprint_error_core(const char *subcommand, const char *format, va_list args, const char *extra)
 {
     fflush(stdout);
+    if (samtools_stdout) hts_flush(samtools_stdout);
+
     if (subcommand && *subcommand) fprintf(stderr, "samtools %s: ", subcommand);
     else fprintf(stderr, "samtools: ");
     vfprintf(stderr, format, args);
@@ -62,6 +74,7 @@ void print_error_errno(const char *subcommand, const char *format, ...)
 
 void check_sam_close(const char *subcmd, samFile *fp, const char *fname, const char *null_fname, int *retp)
 {
+    release_autoflush(fp);
     int r = sam_close(fp);
     if (r >= 0) return;
 
