@@ -364,6 +364,23 @@ class TestParsing(unittest.TestCase):
         self.assertEqual(alleles, [
                          ('T',), ('G', 'A'), ('T', 'A'), ('A', 'G', 'T'), ('GTCT', 'G', 'GTACT')])
 
+    def testAllelesVariantTypes(self):
+        fn = os.path.join(CBCF_DATADIR, self.filename)
+        v = pysam.VariantFile(fn)
+        rec = next(v)
+
+        self.assertEqual(rec.alleles, ('T',))
+        self.assertEqual(rec.alleles_variant_types, ("REF",))
+
+        rec.alleles = ("T", "C")
+        self.assertEqual(rec.alleles_variant_types, ("REF", "SNP"))
+
+        rec.alts = ("TC",)
+        self.assertEqual(rec.alleles_variant_types, ("REF", "INDEL"))
+
+        rec.ref = "AG"
+        self.assertEqual(rec.alleles_variant_types, ("REF", "MNP"))
+
     def testQual(self):
         fn = os.path.join(CBCF_DATADIR, self.filename)
         v = pysam.VariantFile(fn)
@@ -536,8 +553,7 @@ class TestConstructionVCFWithContigs(unittest.TestCase):
         vcf_in = pysam.VariantFile(fn_in)
 
         header = pysam.VariantHeader()
-        for sample in vcf_in.header.samples:
-            header.add_sample(sample)
+        header.add_samples(vcf_in.header.samples)
 
         for hr in vcf_in.header.records:
             header.add_line(str(hr))
@@ -615,10 +631,10 @@ class TestMultiThreading(unittest.TestCase):
 
     def testSingleThreadEqualsMultithreadResult(self):
         with pysam.VariantFile(self.filename) as inf:
-             header = inf.header
-             single = [r for r in inf]
+            header = inf.header
+            single = [r for r in inf]
         with pysam.VariantFile(self.filename, threads=2) as inf:
-             multi = [r for r in inf]
+            multi = [r for r in inf]
         for r1, r2 in zip(single, multi):
             assert str(r1) == str(r2)
 
