@@ -18,12 +18,16 @@ import pysam.samtools
 import pysam.bcftools
 from TestUtils import checkBinaryEqual, check_lines_equal, \
     check_samtools_view_equal, get_temp_filename, force_bytes, WORKDIR, \
-    BAM_DATADIR
+    make_data_files, BAM_DATADIR
 
 
 IS_PYTHON3 = sys.version_info[0] >= 3
 
 
+def setUpModule():
+    make_data_files(BAM_DATADIR)
+
+
 def run_command(cmd):
     '''run a samtools command'''
     try:
@@ -93,14 +97,14 @@ class SamtoolsTest(unittest.TestCase):
         # Samtools-htslib-API: bam_get_library() not yet implemented
         # causes downstream problems
         # TODO: The following cause subsequent commands to fail
-        # unknow option
+        # unknown option
         # "rmdup -s ex1.bam %(out)s_ex1.rmdup.bam",
         # "merge -f %(out)s_ex1.merge.bam ex1.bam ex1.bam",
         "reheader ex2.sam ex1.bam > %(out)s_ex1.reheader.bam",
         "cat -o %(out)s_ex1.cat.bam ex1.bam ex1.bam",
         "targetcut ex1.bam > %(out)s_ex1.targetcut",
         "phase ex1.bam > %(out)s_ex1.phase",
-        "import ex1.fa.fai ex1.sam.gz %(out)s_ex1.bam",
+        "view -bt ex1.fa.fai ex1.sam.gz > %(out)s_ex1.bam",
         "bam2fq ex1.bam > %(out)s_ex1.bam2fq",
         # TODO: not the same
         # "pad2unpad -T ex1.fa ex2.bam > %(out)s_ex2.unpad",
@@ -115,7 +119,7 @@ class SamtoolsTest(unittest.TestCase):
     ]
 
     map_command = {
-        "import": "samimport"}
+    }
 
     executable = "samtools"
 
@@ -242,9 +246,10 @@ class SamtoolsTest(unittest.TestCase):
     def testStatements(self):
         for statement in self.statements:
             command = self.get_command(statement, map_to_internal=False)
-            # bam2fq differs between version 1.5 and 1.6 - reenable if
+            # bam2fq differs between version 1.5 and 1.6 - re-enable if
             # bioconda samtools will be available.
-            if command in ("bedcov", "stats", "dict", "bam2fq"):
+            # flagstat differs between version <=1.12 and >=1.13
+            if command in ("bedcov", "stats", "dict", "bam2fq", "flagstat"):
                 continue
 
             if (command == "calmd" and
@@ -401,13 +406,13 @@ if sys.platform != "darwin":
 #         # "filter -s A ex1.vcf.gz  > %(out)s_ex1.filter",
 #         # exit
 #         # "gtcheck -s A ex1.vcf.gz  > %(out)s_ex1.gtcheck",
-#         # segfauld, used to work wit bcftools 1.3
+#         # segfault, used to work with bcftools 1.3
 #         # "roh -s A ex1.vcf.gz > %(out)s_ex1.roh",
 #         "stats ex1.vcf.gz > %(out)s_ex1.stats",
 #     ]
 
 #     map_command = {
-#         "import": "samimport"}
+#     }
 
 #     executable = "bcftools"
 
@@ -415,9 +420,6 @@ if sys.platform != "darwin":
 
 
 if __name__ == "__main__":
-    # build data files
-    print("building data files")
-    subprocess.call("make -C %s" % BAM_DATADIR, shell=True)
     print("starting tests")
     unittest.main()
     print("completed tests")
