@@ -930,11 +930,11 @@ class TestVCFFromVariantFile(TestVCFFromVCF):
                "filter", "info", "format")
 
     fail_on_parsing = [
-        (24, "Could not parse the header, sample line not found"),
+        (24, 'Could not parse the "#CHROM.." line'),
         ("issue85", "empty VCF"),
     ]
     fail_on_opening = [
-        (24, "Could not parse the header, sample line not found"),
+        (24, 'Could not parse the "#CHROM.." line'),
         ("issue85", "empty VCF"),
     ]
     coordinate_offset = 0
@@ -1024,7 +1024,7 @@ class TestRemoteFileHTTP(unittest.TestCase):
     local = os.path.join(TABIX_DATADIR, "example.gtf.gz")
 
     def setUp(self):
-        if not pysam.config.HAVE_LIBCURL or not check_url(self.url):
+        if not getattr(pysam.config, "HAVE_LIBCURL", 0) or not check_url(self.url):
             self.remote_file = None
         else:
             self.remote_file = pysam.TabixFile(self.url, "r")
@@ -1063,7 +1063,7 @@ class TestRemoteFileHTTPWithHeader(TestRemoteFileHTTP):
     local = os.path.join(TABIX_DATADIR, "example_comments.gtf.gz")
 
     def setUp(self):
-        if not pysam.config.HAVE_LIBCURL or not check_url(self.url):
+        if not getattr(pysam.config, "HAVE_LIBCURL", 0) or not check_url(self.url):
             self.remote_file = None
         else:
             self.remote_file = pysam.TabixFile(self.url, "r")
